unit uWatcher;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls,
  xexecd, xspawn
  ;

type
  TWatcher = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    stProgram: TPanel;
    stStatus: TPanel;
    stExitCode: TPanel;
    stErrorMsg: TPanel;
    btnAction: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnActionClick(Sender: TObject);
  private
    { Private declarations }
    iButtonAction : integer;
    bStopFlag : bool;
    filename : string;
    args : string;
    dir : string;
    nCmdShow : integer;
    wOptions : word;
	procedure 	setActionButton(baFlag : integer);
	procedure	reportFailure(const errorMessage: string);
	procedure	reportStopped;
	procedure	reportSuccess(dwExitCode : longint);
	procedure 	doSpawn;
	procedure	doStop;
	procedure	doClose;
  public
    { Public declarations }
  	constructor Create(AOwner : TComponent;
    					const AFileName : string;
						const AArgs : string;
                        const ADir : string;
                        ACmdShow : integer;
                        AOptions : word);
  end;

var
  Watcher: TWatcher;

implementation

{$R *.DFM}

{Possible values for  iButtonAction }
const
	ba_spawn = 1;
	ba_stop = 2;
	ba_close = 3;

constructor TWatcher.Create(AOwner : TComponent;
    					const AFileName : string;
						const AArgs : string;
                        const ADir : string;
                        ACmdShow : integer;
                        AOptions : word);
begin
    filename := AFilename;
    args := AArgs;
    dir := ADir;
    nCmdShow := ACmdShow;
    wOptions := AOptions;
	inherited Create(AOwner);
end;

procedure TWatcher.FormCreate(Sender: TObject);
begin
    stProgram.Caption := filename;
    stStatus.Caption := 'Click Spawn to start';
    setActionButton(ba_spawn);
end;

procedure TWatcher.setActionButton(baFlag : integer);
begin
	{ Determine what will the action button do when clicked, and what will be }
	{ its caption - Spawn, Stop Tracking or Close }
	iButtonAction := baFlag;
	case (iButtonAction) of
		ba_spawn : btnAction.Caption := '&Spawn';
		ba_stop : btnAction.Caption := '&Stop Tracking';
		ba_close : btnAction.Caption := '&Close';
	end;
end;

procedure TWatcher.btnActionClick(Sender: TObject);
begin
	{ The action button performs one of 3 possible actions, determined }
	{ by the current value of iButtonAction, which is set by the       }
	{ setActionButton routine.                                         }
	case (iButtonAction) of
		ba_spawn : doSpawn;
		ba_stop : doStop;
		ba_close : doClose;
    end; { case }
end;

procedure	TWatcher.reportFailure(const errorMessage: string);
begin
	{ on failure, display 'failure' message and error message }
	stStatus.Caption := 'Failure';
	stExitCode.Caption :=  'N/A';
	stErrorMsg.Caption := errorMessage;
end;

procedure	TWatcher.reportStopped;
begin
	stStatus.Caption := 'Stopped by user';
	stExitCode.Caption := 'N/A';
end;

procedure	TWatcher.reportSuccess(dwExitCode : longint);
begin
	{ on success, display 'success' message and exit code }
	stStatus.Caption := 'Success';
	stExitCode.Caption := IntToStr(dwExitCode);
end;


procedure 	TWatcher.doSpawn;
var
	dwErrorCode,
	dwExitCode : longint;
	hExec : longint;
begin
	stStatus.Caption := 'Spawning program';

    try
        try
            { spawn the program }
            hExec := WXSpawn(filename, args, dir, nCmdShow, wOptions);
            { during wait, action button should stop the }
            { wait }
            setActionButton (ba_stop);
            stStatus.Caption := 'Waiting for termination';
            { Will not return from this call until either }
            { the spawned program terminates or bStopFlag }
            { is set to true (by a click on the action button) }
            bStopFlag := FALSE;
            WXWait(hExec, dwExitCode, bStopFlag, nil);
        except
            on e : EXSpawn do
            begin
                reportFailure(e.message);
                exit;
            end;
        end; { try }

        { if bStopFlag is TRUE, WXWait was terminated because user clicked the }
        { [Stop Tracking] button.											   }
        if (bStopFlag = TRUE) Then
        begin
            WXStopTracking(hExec);
            reportStopped;
        end else
            reportSuccess (dwExitCode);
    finally
		{ action button should now close the form }
		setActionButton (ba_close);
    end; { try }
end;

procedure	TWatcher.doStop;
begin
	{ This is the response to a click on the [Stop Tracking] button. Setting }
	{ bStopFlag to TRUE will cause the WXWait function to abort. }
	bStopFlag := TRUE;
end;

procedure	TWatcher.doClose;
begin
	Close;
end;

end.
